@@ -1,6 +1,6 @@
 /* lispcmds.c -- Lots of standard Lisp functions
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: lispcmds.c,v 1.96 2000/07/14 15:15:36 john Exp $
+   $Id: lispcmds.c,v 1.106 2000/09/24 14:09:14 john Exp $
 
    This file is part of Jade.
 
@@ -35,6 +35,8 @@
 DEFSTRING(default_rep_directory, REP_DIRECTORY);
 DEFSTRING(dot, ".");
 
+static repv default_suffixes;
+
 DEFSYM(or, "or");
 DEFSYM(and, "and");
 DEFSYM(load_path, "load-path");
@@ -91,7 +93,7 @@ set to the name of the file being loaded.
 ::end:: */
 
 DEFUN("quote", Fquote, Squote, (repv args, repv tail_posn), rep_SF) /*
-::doc:quote::
+::doc:rep.lang.interpreter#quote::
 quote ARG
 'ARG
 
@@ -104,7 +106,7 @@ Returns ARG.
 }
 
 DEFUN("lambda", Flambda, Slambda, (repv args, repv tail_posn), rep_SF) /*
-::doc:lambda::
+::doc:rep.lang.interpreter#lambda::
 lambda LAMBDA-LIST BODY...
 
 Evaluates to an anonymous function.
@@ -117,7 +119,7 @@ Evaluates to an anonymous function.
 }
 
 DEFUN("car", Fcar, Scar, (repv cons), rep_Subr1) /*
-::doc:car::
+::doc:rep.data#car::
 car CONS-CELL
 
 Returns the value stored in the car slot of CONS-CELL, or nil if CONS-CELL
@@ -129,7 +131,7 @@ is nil.
     return(Qnil);
 }
 DEFUN("cdr", Fcdr, Scdr, (repv cons), rep_Subr1) /*
-::doc:cdr::
+::doc:rep.data#cdr::
 cdr CONS-CELL
 
 Returns the value stored in the cdr slot of CONS-CELL, or nil if CONS-CELL
@@ -142,7 +144,7 @@ is nil.
 }
 
 DEFUN("list", Flist, Slist, (repv args), rep_SubrN) /*
-::doc:list::
+::doc:rep.data#list::
 list ARGS...
 
 Returns a new list with elements ARGS...
@@ -164,7 +166,7 @@ Returns a new list with elements ARGS...
 }
 
 DEFUN("list*", Flist_star, Slist_star, (repv args), rep_SubrN) /*
-::doc:list*::
+::doc:rep.data#list*::
 list* ARG1 ARG2 ... ARGN
 
 Returns a new list (ARG1 ARG2 ... ARGN-1 . ARGN). That is, the same as from
@@ -192,7 +194,7 @@ Returns a new list (ARG1 ARG2 ... ARGN-1 . ARGN). That is, the same as from
 }
 
 DEFUN("make-list", Fmake_list, Smake_list, (repv len, repv init), rep_Subr2) /*
-::doc:make-list::
+::doc:rep.data#make-list::
 make-list LENGTH [INITIAL-repv]
 
 Returns a new list with LENGTH members, each of which is initialised to
@@ -210,7 +212,7 @@ INITIAL-repv, or nil.
 }
 
 DEFUN("append", Fappend, Sappend, (repv args), rep_SubrN) /*
-::doc:append::
+::doc:rep.data#append::
 append LISTS...
 
 Non-destructively concatenates each of it's argument LISTS... into one
@@ -246,7 +248,7 @@ new list which is returned.
 }
 
 DEFUN("nconc", Fnconc, Snconc, (repv args), rep_SubrN) /*
-::doc:nconc::
+::doc:rep.data#nconc::
 nconc LISTS...
 
 Destructively concatenates each of it's argument LISTS... into one new
@@ -284,7 +286,7 @@ to the beginning of the next list. Returns the new list.
 }
 
 DEFUN("rplaca", Frplaca, Srplaca, (repv cons, repv car), rep_Subr2) /*
-::doc:rplaca::
+::doc:rep.data#rplaca::
 rplaca CONS-CELL NEW-CAR
 
 Sets the value of the car slot in CONS-CELL to NEW-CAR.
@@ -299,7 +301,7 @@ Returns the CONS-CELL.
 }
 
 DEFUN("rplacd", Frplacd, Srplacd, (repv cons, repv cdr), rep_Subr2) /*
-::doc:rplacd::
+::doc:rep.data#rplacd::
 rplacd CONS-CELL NEW-CDR
 
 Sets the value of the cdr slot in CONS-CELL to NEW-CDR.
@@ -314,7 +316,7 @@ Returns the CONS-CELL.
 }
 
 DEFUN("reverse", Freverse, Sreverse, (repv head), rep_Subr1) /*
-::doc:reverse::
+::doc:rep.data#reverse::
 reverse LIST
 
 Returns a new list which is a copy of LIST except that the members are in
@@ -335,7 +337,7 @@ reverse order.
 }
 
 DEFUN("nreverse", Fnreverse, Snreverse, (repv head), rep_Subr1) /*
-::doc:nreverse::
+::doc:rep.data#nreverse::
 nreverse LIST
 
 Returns LIST altered so that it's members are in reverse order to what they
@@ -364,7 +366,7 @@ were. This function is destructive towards it's argument.
 }
 
 DEFUN("assoc", Fassoc, Sassoc, (repv elt, repv list), rep_Subr2) /*
-::doc:assoc::
+::doc:rep.data#assoc::
 assoc ELT ASSOC-LIST
 
 Searches ASSOC-LIST for a list whose first element is ELT. `assoc' uses
@@ -390,7 +392,7 @@ For example,
 }
 
 DEFUN("assq", Fassq, Sassq, (repv elt, repv list), rep_Subr2) /*
-::doc:assq::
+::doc:rep.data#assq::
 assq ELT ASSOC-LIST
 
 Searches ASSOC-LIST for a list whose first element is ELT. `assq' uses `eq'
@@ -413,7 +415,7 @@ association.
 }
 
 DEFUN("rassoc", Frassoc, Srassoc, (repv elt, repv list), rep_Subr2) /*
-::doc:rassoc::
+::doc:rep.data#rassoc::
 rassoc ELT ASSOC-LIST
 
 Searches ASSOC-LIST for a cons-cell whose cdr element is `equal' to ELT. 
@@ -438,7 +440,7 @@ For example,
 }
 
 DEFUN("rassq", Frassq, Srassq, (repv elt, repv list), rep_Subr2) /*
-::doc:rassq::
+::doc:rep.data#rassq::
 rassq ELT ASSOC-LIST
 
 Searches ASSOC-LIST for a cons-cell whose cdr is `eq' to ELT.
@@ -460,7 +462,7 @@ Returns the first matching cons-cell, else nil.
 }
 
 DEFUN("nth", Fnth, Snth, (repv index, repv list), rep_Subr2) /*
-::doc:nth::
+::doc:rep.data#nth::
 nth INDEX LIST
 
 Returns the INDEXth element of LIST. The first element has an INDEX of zero.
@@ -483,7 +485,7 @@ Returns the INDEXth element of LIST. The first element has an INDEX of zero.
 }
 
 DEFUN("nthcdr", Fnthcdr, Snthcdr, (repv index, repv list), rep_Subr2) /*
-::doc:nthcdr::
+::doc:rep.data#nthcdr::
 nthcdr INDEX LIST
 
 Returns the INDEXth cdr of LIST. The first is INDEX zero.
@@ -506,7 +508,7 @@ Returns the INDEXth cdr of LIST. The first is INDEX zero.
 }
 
 DEFUN("last", Flast, Slast, (repv list), rep_Subr1) /*
-::doc:last::
+::doc:rep.data#last::
 last LIST
 
 Returns the last element of LIST.
@@ -528,7 +530,7 @@ Returns the last element of LIST.
 }
 
 DEFUN("mapcar", Fmapcar, Smapcar, (repv fun, repv list), rep_Subr2) /*
-::doc:mapcar::
+::doc:rep.data#mapcar::
 mapcar FUNCTION LIST
 
 Calls FUNCTION-NAME with each element of LIST as an argument in turn and
@@ -563,7 +565,7 @@ returns a new list constructed from the results, ie,
 }
 
 DEFUN("mapc", Fmapc, Smapc, (repv fun, repv list), rep_Subr2) /*
-::doc:mapc::
+::doc:rep.data#mapc::
 mapc FUNCTION LIST
 
 Applies FUNCTION to each element in LIST, discards the results.
@@ -586,7 +588,7 @@ Applies FUNCTION to each element in LIST, discards the results.
 }
 
 DEFUN("filter", Ffilter, Sfilter, (repv pred, repv list), rep_Subr2) /*
-::doc:filter::
+::doc:rep.data#filter::
 filter PREDICATE LIST
 
 Return a new list, consisting of the elements in LIST which the function
@@ -625,7 +627,7 @@ PREDICATE returns t when applied to; i.e. something like
 }
 
 DEFUN("member", Fmember, Smember, (repv elt, repv list), rep_Subr2) /*
-::doc:member::
+::doc:rep.data#member::
 member ELT LIST
 
 If ELT is a member of list LIST then return the tail of the list starting
@@ -649,7 +651,7 @@ from the matched ELT, ie,
 }
 
 DEFUN("memq", Fmemq, Smemq, (repv elt, repv list), rep_Subr2) /*
-::doc:memq::
+::doc:rep.data#memq::
 memq ELT LIST
 
 If ELT is a member of list LIST then return the tail of the list starting
@@ -673,7 +675,7 @@ from the matched ELT, ie,
 }
 
 DEFUN("memql", Fmemql, Smemql, (repv elt, repv list), rep_Subr2) /*
-::doc:memql::
+::doc:rep.data#memql::
 memql ELT LIST
 
 If ELT is a member of list LIST then return the tail of the list starting
@@ -700,7 +702,7 @@ from the matched ELT. `memql' uses `eql' to compare list items.
 }
 
 DEFUN("delete", Fdelete, Sdelete, (repv elt, repv list), rep_Subr2) /*
-::doc:delete::
+::doc:rep.data#delete::
 delete ELT LIST
 
 Returns LIST with any members `equal' to ELT destructively removed.
@@ -722,7 +724,7 @@ Returns LIST with any members `equal' to ELT destructively removed.
 }
 
 DEFUN("delq", Fdelq, Sdelq, (repv elt, repv list), rep_Subr2) /*
-::doc:delq::
+::doc:rep.data#delq::
 delq ELT LIST
 
 Returns LIST with any members `eq' to ELT destructively removed.
@@ -744,7 +746,7 @@ Returns LIST with any members `eq' to ELT destructively removed.
 }
 
 DEFUN("delete-if", Fdelete_if, Sdelete_if, (repv pred, repv list), rep_Subr2) /*
-::doc:delete-if::
+::doc:rep.data#delete-if::
 delete-if FUNCTION LIST
 
 Similar to `delete' except that a predicate function, FUNCTION-NAME, is
@@ -779,7 +781,7 @@ applied to that element, ie,
 }
 
 DEFUN("delete-if-not", Fdelete_if_not, Sdelete_if_not, (repv pred, repv list), rep_Subr2) /*
-::doc:delete-if-not::
+::doc:rep.data#delete-if-not::
 delete-if-not FUNCTION LIST
 
 Similar to `delete' except that a predicate function, FUNCTION-NAME, is
@@ -814,7 +816,7 @@ applied to that element, ie,
 }
 
 DEFUN("vector", Fvector, Svector, (repv args), rep_SubrN) /*
-::doc:vector::
+::doc:rep.data#vector::
 vector ARGS...
 
 Returns a new vector with ARGS... as its elements.
@@ -838,7 +840,7 @@ Returns a new vector with ARGS... as its elements.
 }
 
 DEFUN("make-vector", Fmake_vector, Smake_vector, (repv size, repv init), rep_Subr2) /*
-::doc:make-vector::
+::doc:rep.data#make-vector::
 make-vector SIZE [INITIAL-repv]
 
 Creates a new vector of size SIZE. If INITIAL-repv is provided each element
@@ -862,7 +864,7 @@ will be set to that value, else they will all be nil.
 }
 
 DEFUN("arrayp", Farrayp, Sarrayp, (repv arg), rep_Subr1) /*
-::doc:arrayp::
+::doc:rep.data#arrayp::
 arrayp ARG
 
 Returns t when ARG is an array.
@@ -872,7 +874,7 @@ Returns t when ARG is an array.
 }
 
 DEFUN("aset", Faset, Saset, (repv array, repv index, repv new), rep_Subr3) /*
-::doc:aset::
+::doc:rep.data#aset::
 aset ARRAY INDEX NEW-VALUE
 
 Sets element number INDEX (a positive integer) of ARRAY (can be a vector
@@ -891,6 +893,7 @@ can only contain characters (ie, integers).
 	{
 	    rep_DECLARE3(new, rep_INTP);
 	    rep_STR(array)[rep_INT(index)] = (u_char)rep_INT(new);
+	    rep_string_modified (array);
 	    return(new);
 	}
     }
@@ -910,7 +913,7 @@ can only contain characters (ie, integers).
 }
 
 DEFUN("aref", Faref, Saref, (repv array, repv index), rep_Subr2) /*
-::doc:aref::
+::doc:rep.data#aref::
 aref ARRAY INDEX
 
 Returns the INDEXth (a non-negative integer) element of ARRAY, which
@@ -936,7 +939,7 @@ can be a vector or a string. INDEX starts at zero.
 }
 
 DEFUN("make-string", Fmake_string, Smake_string, (repv len, repv init), rep_Subr2) /*
-::doc:make-string::
+::doc:rep.data#make-string::
 make-string LENGTH [INITIAL-repv]
 
 Returns a new string of length LENGTH, each character is initialised to
@@ -957,7 +960,7 @@ INITIAL-repv, or to space if INITIAL-repv is not given.
 }
 
 DEFUN("substring", Fsubstring, Ssubstring, (repv string, repv start, repv end), rep_Subr3) /*
-::doc:substring::
+::doc:rep.data#substring::
 substring STRING START [END]
 
 Returns the portion of STRING starting at character number START and ending
@@ -1002,7 +1005,7 @@ extend_concat(u_char **buf, int *bufLen, int i, int addLen)
     return(rep_FALSE);
 }
 DEFUN("concat", Fconcat, Sconcat, (repv args), rep_SubrN) /*
-::doc:concat::
+::doc:rep.data#concat::
 concat ARGS...
 
 Concatenates all ARGS... into a single string, each argument can be a string,
@@ -1019,58 +1022,62 @@ a character or a list or vector of characters.
 	while(rep_CONSP(args))
 	{
 	    repv arg = rep_CAR(args);
-	    switch(rep_TYPE(arg))
+	    if (arg != Qnil)
 	    {
-		int slen, j;
-	    case rep_String:
-		slen = rep_STRING_LEN(arg);
-		if(!extend_concat(&buf, &buflen, i, slen))
-		    goto error;
-		memcpy(buf + i, rep_STR(arg), slen);
-		i += slen;
-		break;
-	    case rep_Int:
-		if(!extend_concat(&buf, &buflen, i, 1))
-		    goto error;
-		buf[i++] = rep_INT(arg);
-		break;
-	    case rep_Symbol:
-		if(arg != Qnil)
-		    break;
-		/* FALL THROUGH */
-	    case rep_Cons:
-		while(rep_CONSP(arg))
+		switch(rep_TYPE(arg))
 		{
-		    repv ch = rep_CAR(arg);
-		    if(rep_INTP(ch))
-		    {
-			if(!extend_concat(&buf, &buflen, i, 1))
-			    goto error;
-			buf[i++] = rep_INT(ch);
-		    }
-		    arg = rep_CDR(arg);
-		    rep_TEST_INT;
-		    if(rep_INTERRUPTP)
+		    int slen, j;
+
+		case rep_String:
+		    slen = rep_STRING_LEN(arg);
+		    if(!extend_concat(&buf, &buflen, i, slen))
 			goto error;
-		}
-		break;
-	    case rep_Vector:
-		{
-		    int len = rep_VECT_LEN(arg);
-		    for(j = 0; j < len; j++)
+		    memcpy(buf + i, rep_STR(arg), slen);
+		    i += slen;
+		    break;
+
+		case rep_Int:
+		    if(!extend_concat(&buf, &buflen, i, 1))
+			goto error;
+		    buf[i++] = rep_INT(arg);
+		    break;
+
+		case rep_Symbol:
+		case rep_Cons:
+		    while(rep_CONSP(arg))
 		    {
-			if(rep_INTP(rep_VECTI(arg, j)))
+			repv ch = rep_CAR(arg);
+			if(rep_INTP(ch))
 			{
 			    if(!extend_concat(&buf, &buflen, i, 1))
 				goto error;
-			    buf[i++] = rep_INT(rep_VECTI(arg, j));
+			    buf[i++] = rep_INT(ch);
 			}
+			arg = rep_CDR(arg);
+			rep_TEST_INT;
+			if(rep_INTERRUPTP)
+			    goto error;
 		    }
 		    break;
+
+		case rep_Vector:
+		    {
+			int len = rep_VECT_LEN(arg);
+			for(j = 0; j < len; j++)
+			{
+			    if(rep_INTP(rep_VECTI(arg, j)))
+			    {
+				if(!extend_concat(&buf, &buflen, i, 1))
+				    goto error;
+				buf[i++] = rep_INT(rep_VECTI(arg, j));
+			    }
+			}
+			break;
+		    }
+		default:
+		    res = rep_signal_arg_error(arg, argnum);
+		    goto error;
 		}
-	    default:
-		res = rep_signal_arg_error(arg, argnum);
-		goto error;
 	    }
 	    args = rep_CDR(args);
 	    argnum++;
@@ -1084,12 +1091,15 @@ error:
 }
 
 DEFUN("length", Flength, Slength, (repv sequence), rep_Subr1) /*
-::doc:length::
+::doc:rep.data#length::
 length SEQUENCE
 
 Returns the number of elements in SEQUENCE (a string, list or vector).
 ::end:: */
 {
+    if (sequence == Qnil)
+	return rep_MAKE_INT (0);
+
     switch(rep_TYPE(sequence))
     {
 	int i;
@@ -1111,29 +1121,23 @@ Returns the number of elements in SEQUENCE (a string, list or vector).
 	}
 	return(rep_MAKE_INT(i));
 	break;
-    case rep_Symbol:
-	if(sequence == Qnil)
-	    return(rep_MAKE_INT(0));
-	/* FALL THROUGH */
     default:
 	return rep_signal_arg_error (sequence, 1);
     }
 }
 
 DEFUN("copy-sequence", Fcopy_sequence, Scopy_sequence, (repv seq), rep_Subr1) /*
-::doc:copy-sequence::
+::doc:rep.data#copy-sequence::
 copy-sequence SEQUENCE
 
 Returns a new sequence whose elements are eq to those in SEQUENCE.
 ::end:: */
 {
     repv res = Qnil;
+    if (seq == Qnil)
+	return Qnil;
     switch(rep_TYPE(seq))
     {
-    case rep_Symbol:
-	if(!rep_NILP(seq))
-	    res = rep_signal_arg_error(seq, 1);
-	break;
     case rep_Cons:
 	{
 	    repv *last = &res;
@@ -1169,7 +1173,7 @@ Returns a new sequence whose elements are eq to those in SEQUENCE.
 }
 
 DEFUN("elt", Felt, Selt, (repv seq, repv index), rep_Subr2) /*
-::doc:elt::
+::doc:rep.data#elt::
 elt SEQUENCE INDEX
 
 Return the element of SEQUENCE at position INDEX (counting from zero).
@@ -1182,7 +1186,7 @@ Return the element of SEQUENCE at position INDEX (counting from zero).
 }
 
 DEFUN("cond", Fcond, Scond, (repv args, repv tail_posn), rep_SF) /*
-::doc:cond::
+::doc:rep.lang.interpreter#cond::
 cond (CONDITION FORMS... ) ...
 
 Find the first CONDITION which has a value of t when eval'ed, then perform
@@ -1225,94 +1229,6 @@ like the last else in an else-if statement in C.
     return(res);
 }
 
-DEFUN("case", Fcase, Scase, (repv args, repv tail_posn), rep_SF) /*
-::doc:case::
-case KEY CLAUSES...
-
-Each CLAUSE is `((ITEMS... ) FORMS...)'. Find the first CLAUSE with an
-ITEM matching (using `eql') the result of evaluating KEY (only
-evaluated once), then evaluate the associated FORMS in a `progn'. The
-final clause may have the form `(t FORMS...)', which always matches KEY
-if no other CLAUSE has already. Returns `nil' if no clause matches.
-
-If any of the ITEMS appear more than once, then the behaviour is
-undefined.
-::end:: */
-{
-    repv key;
-    rep_GC_root gc_args;
-    int i = 2;
-
-    if (!rep_CONSP(args))
-	return rep_signal_missing_arg (0);
-    
-    rep_PUSHGC(gc_args, args);
-    key = rep_eval (rep_CAR(args), Qnil);
-    rep_POPGC;
-    if (key == rep_NULL)
-	return rep_NULL;
-    args = rep_CDR(args);
-
-    while(rep_CONSP(args))
-    {
-	if (!rep_CONSP(rep_CAR(args)))
-	    return rep_signal_arg_error (rep_CAR(args), i);
-	if (rep_CONSP(rep_CAAR(args)))
-	{
-	    repv ptr = rep_CAAR(args);
-	    while (rep_CONSP(ptr))
-	    {
-		if (Feql (key, rep_CAR(ptr)) != Qnil)
-		    return Fprogn (rep_CDAR(args), tail_posn);
-		ptr = rep_CDR(ptr);
-		rep_TEST_INT;
-		if (rep_INTERRUPTP)
-		    return rep_NULL;
-	    }
-	}
-	else if (rep_CAAR(args) == Qt)
-	    return Fprogn (rep_CDAR(args), tail_posn);
-	else
-	    return rep_signal_arg_error (rep_CAR(args), i);
-	args = rep_CDR(args);
-	i++;
-    }
-    return Qnil;
-}
-
-DEFUN("apply", Fapply, Sapply, (repv args), rep_SubrN) /*
-::doc:apply::
-apply FUNCTION ARGS... ARG-LIST
-
-Calls FUNCTION passing all of ARGS to it as well as all elements in ARG-LIST.
-ie,
-  (apply '+ 1 2 3 '(4 5 6))
-   => 21
-::end:: */
-{
-    repv list = Qnil, *last;
-    last = &list;
-    if(rep_CONSP(args))
-    {
-	while(rep_CONSP(rep_CDR(args)))
-	{
-	    if(!(*last = Fcons(rep_CAR(args), Qnil)))
-		return(rep_NULL);
-	    last = &rep_CDR(*last);
-	    args = rep_CDR(args);
-	    rep_TEST_INT;
-	    if(rep_INTERRUPTP)
-		return(rep_NULL);
-	}
-	if(!rep_NILP(Flistp(rep_CAR(args))))
-	    *last = rep_CAR(args);
-	else
-	    return rep_signal_arg_error (rep_CAR (args), -1);
-	return(Ffuncall(list));
-    }
-    return rep_signal_missing_arg(1);
-}
-
 static inline repv
 load_file_exists_p (repv name)
 {
@@ -1326,8 +1242,109 @@ load_file_exists_p (repv name)
     return tem;
 }
 
+DEFUN ("load-file", Fload_file, Sload_file,
+       (repv name, repv structure), rep_Subr2) /*
+::doc:rep.io.files#load-file::
+load-file FILENAME [STRUCTURE]
+
+Load the file of Lisp forms called FILENAME (no suffixes are added, or
+paths searched). The file is loaded in a null lexical environment,
+within STRUCTURE. The value of the last form evaluated is returned.
+::end:: */
+{
+    repv stream, bindings = Qnil, result, tem;
+    rep_GC_root gc_stream, gc_bindings;
+    struct rep_Call lc;
+    int c;
+
+    if (structure == Qnil)
+	structure = rep_structure;
+
+    rep_DECLARE1 (name, rep_STRINGP);
+    rep_DECLARE2 (structure, rep_STRUCTUREP);
+
+    rep_PUSHGC (gc_stream, name);
+    rep_PUSHGC (gc_bindings, structure);
+    stream = Fopen_file (name, Qread);
+    rep_POPGC; rep_POPGC;
+    if (!stream || !rep_FILEP (stream))
+	return rep_NULL;
+
+    bindings = rep_bind_symbol (bindings, Qload_filename, name);
+    rep_PUSHGC (gc_stream, stream);
+    rep_PUSHGC (gc_bindings, bindings);
+
+    /* Create the lexical environment for the file. */
+    lc.fun = Qnil;
+    lc.args = Qnil;
+    lc.args_evalled_p = Qnil;
+    rep_PUSH_CALL (lc);
+    rep_env = Qnil;
+    rep_structure = structure;
+
+    result = Qnil;
+    c = rep_stream_getc (stream);
+    while ((c != EOF) && (tem = rep_readl (stream, &c)))
+    {
+	rep_TEST_INT;
+	if (rep_INTERRUPTP || !(result = rep_eval (tem, Qnil)))
+	{
+	    result = rep_NULL;
+	    goto out;
+	}
+    }
+    if (rep_throw_value
+	&& rep_CAR (rep_throw_value) == Qerror
+	&& rep_CONSP (rep_CDR(rep_throw_value))
+	&& rep_CAR (rep_CDR(rep_throw_value)) == Qend_of_stream)
+    {
+	/* lose the end-of-stream error. */
+	rep_throw_value = rep_NULL;
+    }
+out:
+    rep_POP_CALL (lc);
+    rep_POPGC; rep_POPGC;
+
+    rep_PUSHGC (gc_stream, result);
+    rep_unbind_symbols (bindings);
+    Fclose_file (stream);
+    rep_POPGC;
+
+    return result;
+}
+
+DEFUN ("load-dl-file", Fload_dl_file, Sload_dl_file,
+       (repv name, repv structure), rep_Subr2)
+{
+    struct rep_Call lc;
+    repv result;
+
+    if (structure == Qnil)
+	structure = rep_structure;
+
+    rep_DECLARE1 (name, rep_STRINGP);
+    rep_DECLARE2 (structure, rep_STRUCTUREP);
+
+    /* Create the lexical environment for the file. */
+    lc.fun = Qnil;
+    lc.args = Qnil;
+    lc.args_evalled_p = Qnil;
+    rep_PUSH_CALL (lc);
+    rep_env = Qnil;
+    rep_structure = structure;
+
+#ifdef HAVE_DYNAMIC_LOADING
+    result = rep_open_dl_library (name);
+#else
+    result = Fsignal (Qerror, rep_LIST_1 (rep_string_dup ("No support for dynamic loading of shared libraries")));
+#endif
+
+    rep_POP_CALL (lc);
+    return result;
+}
+
 DEFUN_INT("load", Fload, Sload, (repv file, repv noerr_p, repv nopath_p, repv nosuf_p, repv unused), rep_Subr5, "fLisp file to load:") /*
-::doc:load::
+::doc:rep.io.files#load::
 load FILE [NO-ERROR] [NO-PATH] [NO-SUFFIX]
 
 Attempt to open and then read-and-eval the file of Lisp code FILE.
@@ -1353,10 +1370,7 @@ loaded and a warning is displayed.
     repv dir = rep_NULL, try = rep_NULL;
     repv result = rep_NULL;
     repv suffixes;
-
-#ifdef HAVE_DYNAMIC_LOADING
     rep_bool trying_dl = rep_FALSE;
-#endif
 
     rep_GC_root gc_file, gc_name, gc_path, gc_dir, gc_try, gc_result, gc_suffixes;
 
@@ -1372,7 +1386,7 @@ loaded and a warning is displayed.
 
     suffixes = F_structure_ref (rep_structure, Q_load_suffixes);
     if (!suffixes || !rep_CONSP (suffixes))
-	no_suffix_p = rep_TRUE;
+	suffixes = default_suffixes;
 
     rep_PUSHGC(gc_name, name);
     rep_PUSHGC(gc_file, file);
@@ -1485,77 +1499,22 @@ path_error:
 	    return Qnil;
     }
 
+    rep_PUSHGC (gc_file, file);
 #ifdef HAVE_DYNAMIC_LOADING
     if(trying_dl)
-    {
-	rep_PUSHGC(gc_file, file);
-	result = rep_open_dl_library (name);
-	rep_POPGC;
-	if (result == rep_NULL)
-	    return rep_NULL;
-    }
+	result = Fload_dl_file (name, rep_structure);
     else
 #endif
-    {
-	repv stream, bindings = Qnil;
-	rep_GC_root gc_stream, gc_bindings;
-	struct rep_Call lc;
-
-	repv tem;
-	int c;
-
-	rep_PUSHGC(gc_file, file);
-	rep_PUSHGC(gc_stream, name);
-	stream = Fopen_file(name, Qread);
-	rep_POPGC;
-	if(!stream || !rep_FILEP(stream))
-	{
-	    rep_POPGC;
-       	    return rep_NULL;
-	}
-
-	bindings = rep_bind_symbol (bindings, Qload_filename, name);
-	rep_PUSHGC(gc_stream, stream);
-	rep_PUSHGC(gc_bindings, bindings);
-
-	/* Create the lexical environment for the file. */
-	lc.fun = Qnil;
-	lc.args = Qnil;
-	lc.args_evalled_p = Qnil;
-	rep_PUSH_CALL(lc);
-	rep_env = Qnil;
-
-	result = Qnil;
-	c = rep_stream_getc(stream);
-	while((c != EOF) && (tem = rep_readl(stream, &c)))
-	{
-	    rep_TEST_INT;
-	    if(rep_INTERRUPTP || !(result = rep_eval(tem, Qnil)))
-	    {
-		rep_unbind_symbols (bindings);
-		rep_POP_CALL(lc);
-		rep_POPGC; rep_POPGC; rep_POPGC;
-		return rep_NULL;
-	    }
-	}
-	if (rep_throw_value
-	    && rep_CAR(rep_throw_value) == Qerror
-	    && rep_CONSP(rep_CDR(rep_throw_value))
-	    && rep_CAR(rep_CDR(rep_throw_value)) == Qend_of_stream)
-	{
-	    /* lose the end-of-stream error. */
-	    rep_throw_value = rep_NULL;
-	}
-	rep_POP_CALL(lc);
-	rep_POPGC; rep_POPGC;
-	rep_PUSHGC (gc_result, result);
-	rep_unbind_symbols (bindings);
-	Fclose_file (stream);
-	rep_POPGC; rep_POPGC;
-    }
+	result = Fload_file (name, rep_structure);
+    rep_POPGC;
+    if (result == rep_NULL)
+	return rep_NULL;
 
     /* Loading succeeded. Look for an applicable item in
        the after-load-alist. */
+    if (rep_STRUCTUREP (result) && rep_STRUCTURE (result)->name != Qnil)
+	/* use the canonical name in case of aliasing.. */
+	file = rep_SYM (rep_STRUCTURE (result)->name)->name;
     rep_PUSHGC (gc_result, result);
     rep_PUSHGC (gc_file, file);
     {
@@ -1594,7 +1553,7 @@ again:
 }
 
 DEFUN("equal", Fequal, Sequal, (repv val1, repv val2), rep_Subr2) /*
-::doc:equal::
+::doc:rep.data#equal::
 equal VALUE1 VALUE2
 
 Compares VALUE1 and VALUE2, compares the actual structure of the objects not
@@ -1607,7 +1566,7 @@ location in memory is different.
 }
 
 DEFUN("eq", Feq, Seq, (repv val1, repv val2), rep_Subr2) /*
-::doc:eq::
+::doc:rep.data#eq::
 eq VALUE1 VALUE2
 
 Returns t if VALUE1 and VALUE2 are one and the same object. Note that
@@ -1618,7 +1577,7 @@ this may or may not be true for numbers of the same value (see `eql').
 }
 
 DEFUN("not", Fnot, Snot, (repv arg), rep_Subr1) /*
-::doc:not::
+::doc:rep.data#not::
 not ARG
 
 If ARG is nil returns t, else returns nil.
@@ -1630,7 +1589,7 @@ If ARG is nil returns t, else returns nil.
 }
 
 DEFUN("string-head-eq", Fstring_head_eq, Sstring_head_eq, (repv str1, repv str2), rep_Subr2) /*
-::doc:string-head-eq::
+::doc:rep.data#string-head-eq::
 string-head-eq STRING1 STRING2
 
 Returns t if STRING2 matches the beginning of STRING1, ie,
@@ -1656,7 +1615,7 @@ Returns t if STRING2 matches the beginning of STRING1, ie,
 }
 
 DEFUN("string-equal", Fstring_equal, Sstring_equal, (repv str1, repv str2), rep_Subr2) /*
-::doc:string-equal::
+::doc:rep.data#string-equal::
 string-equal STRING1 STRING2
 
 Returns t if STRING1 and STRING2 are the same, ignoring case.
@@ -1677,7 +1636,7 @@ Returns t if STRING1 and STRING2 are the same, ignoring case.
 }
 
 DEFUN("string-lessp", Fstring_lessp, Sstring_lessp, (repv str1, repv str2), rep_Subr2) /*
-::doc:string-lessp::
+::doc:rep.data#string-lessp::
 string-lessp STRING1 STRING2
 
 Returns t if STRING1 is `less' than STRING2, ignoring case.
@@ -1726,7 +1685,7 @@ Returns t if STRING1 is `less' than STRING2, ignoring case.
     return rep_signal_missing_arg(rep_CONSP(args) ? 2 : 1);
 
 DEFUN("=", Fnum_eq, Snum_eq, (repv args), rep_SubrN) /*
-::doc:=::
+::doc:rep.data#=::
 = ARG1 ARG2 [ARG3 ...]
 
 Returns t if each value is the same as every other value. (Using
@@ -1738,7 +1697,7 @@ ignored.)
 }
 
 DEFUN("/=", Fnum_noteq, Snum_noteq, (repv args), rep_SubrN) /*
-::doc::/=::
+::doc:rep.data#:/=::
 /= ARG1 ARG2 ...
 
 Returns t if each value is different from every other value. (Using
@@ -1754,7 +1713,7 @@ ignored.)
 }
 
 DEFUN(">", Fgtthan, Sgtthan, (repv args), rep_SubrN) /*
-::doc:>::
+::doc:rep.data#>::
 > ARG1 ARG2 [ARG3 ...]
 
 Returns t if ARG1 is greater than ARG2, and if ARG2 is greater than ARG3,
@@ -1766,7 +1725,7 @@ strings, positions, marks, etc as well.
 }
 
 DEFUN(">=", Fgethan, Sgethan, (repv args), rep_SubrN) /*
-::doc:>=::
+::doc:rep.data#>=::
 >= ARG1 ARG2 [ARG3 ...]
 
 Returns t if ARG1 is greater-or-equal than ARG2. Note that this command
@@ -1777,7 +1736,7 @@ isn't limited to numbers, it can do strings, positions, marks, etc as well.
 }
 
 DEFUN("<", Fltthan, Sltthan, (repv args), rep_SubrN) /*
-::doc:<::
+::doc:rep.data#<::
 < ARG1 ARG2 [ARG3 ...]
 
 Returns t if ARG1 is less than ARG2. Note that this command isn't limited to
@@ -1788,7 +1747,7 @@ numbers, it can do strings, positions, marks, etc as well.
 }
 
 DEFUN("<=", Flethan, Slethan, (repv args), rep_SubrN) /*
-::doc:<=::
+::doc:rep.data#<=::
 <= ARG1 ARG2 [ARG3 ...]
 
 Returns t if ARG1 is less-or-equal than ARG2. Note that this command isn't
@@ -1799,7 +1758,7 @@ limited to numbers, it can do strings, positions, marks, etc as well.
 }
 
 DEFUN("null", Fnull, Snull, (repv arg), rep_Subr1) /*
-::doc:null::
+::doc:rep.data#null::
 null ARG
 
 Returns t if ARG is nil.
@@ -1809,7 +1768,7 @@ Returns t if ARG is nil.
 }
 
 DEFUN("atom", Fatom, Satom, (repv arg), rep_Subr1) /*
-::doc:atom::
+::doc:rep.data#atom::
 atom ARG
 
 Returns t if ARG is not a cons-cell.
@@ -1819,7 +1778,7 @@ Returns t if ARG is not a cons-cell.
 }
 
 DEFUN("consp", Fconsp, Sconsp, (repv arg), rep_Subr1) /*
-::doc:consp::
+::doc:rep.data#consp::
 consp ARG
 
 Returns t if ARG is a cons-cell.
@@ -1829,7 +1788,7 @@ Returns t if ARG is a cons-cell.
 }
 
 DEFUN("listp", Flistp, Slistp, (repv arg), rep_Subr1) /*
-::doc:listp::
+::doc:rep.data#listp::
 listp ARG
 
 Returns t if ARG is a list, (either a cons-cell or nil).
@@ -1839,7 +1798,7 @@ Returns t if ARG is a list, (either a cons-cell or nil).
 }
 
 DEFUN("stringp", Fstringp, Sstringp, (repv arg), rep_Subr1) /*
-::doc:stringp::
+::doc:rep.data#stringp::
 stringp ARG
 
 Returns t is ARG is a string.
@@ -1849,7 +1808,7 @@ Returns t is ARG is a string.
 }
 
 DEFUN("vectorp", Fvectorp, Svectorp, (repv arg), rep_Subr1) /*
-::doc:vectorp::
+::doc:rep.data#vectorp::
 vectorp ARG
 
 Returns t if ARG is a vector.
@@ -1858,18 +1817,8 @@ Returns t if ARG is a vector.
     return rep_VECTORP(arg) ? Qt : Qnil;
 }
 
-DEFUN("bytecodep", Fbytecodep, Sbytecodep, (repv arg), rep_Subr1) /*
-::doc:bytecodep::
-bytecodep ARG
-
-Returns t if ARG is a byte code subroutine (i.e. compiled Lisp code).
-::end:: */
-{
-    return rep_COMPILEDP(arg) ? Qt : Qnil;
-}
-
 DEFUN("functionp", Ffunctionp, Sfunctionp, (repv arg), rep_Subr1) /*
-::doc:functionp::
+::doc:rep.lang.interpreter#functionp::
 functionp ARG
 
 Returns t if ARG is a function.
@@ -1899,7 +1848,7 @@ Returns t if ARG is a function.
 }
 
 DEFUN("macrop", Fmacrop, Smacrop, (repv arg), rep_Subr1) /*
-::doc:macrop::
+::doc:rep.lang.interpreter#macrop::
 macrop ARG
 
 Returns t if ARG is a macro.
@@ -1912,7 +1861,7 @@ Returns t if ARG is a macro.
 }
 	
 DEFUN("special-form-p", Fspecial_form_p, Sspecial_form_p, (repv arg), rep_Subr1) /*
-::doc:special-form-p::
+::doc:rep.lang.interpreter#special-form-p::
 special-form-p ARG
 
 Returns t if ARG is a special-form.
@@ -1924,7 +1873,7 @@ Returns t if ARG is a special-form.
 }
 
 DEFUN("subrp", Fsubrp, Ssubrp, (repv arg), rep_Subr1) /*
-::doc:subrp::
+::doc:rep.lang.interpreter#subrp::
 subrp ARG
 
 Returns t if arg is a primitive function.
@@ -1947,7 +1896,7 @@ Returns t if arg is a primitive function.
 }
 
 DEFUN("sequencep", Fsequencep, Ssequencep, (repv arg), rep_Subr1) /*
-::doc:sequencep::
+::doc:rep.data#sequencep::
 sequencep ARG
 
 Returns t is ARG is a sequence (a list, vector or string).
@@ -1960,7 +1909,7 @@ Returns t is ARG is a sequence (a list, vector or string).
 }
 
 DEFUN("subr-name", Fsubr_name, Ssubr_name, (repv subr, repv useVar), rep_Subr2) /*
-::doc:subr-name::
+::doc:rep.lang.interpreter#subr-name::
 subr-name SUBR [USE-VAR]
 
 Returns the name (a string) associated with SUBR.
@@ -1983,7 +1932,7 @@ Returns the name (a string) associated with SUBR.
 }
 
 DEFUN("call-hook", Fcall_hook, Scall_hook, (repv hook, repv arg_list, repv type), rep_Subr3) /*
-::doc:call-hook::
+::doc:rep.system#call-hook::
 call-hook HOOK ARG-LIST [TYPE]
 
 Call the hook named by the symbol HOOK, passing all functions the arguments
@@ -2031,7 +1980,7 @@ returned.
 DEFUN("call-with-exception-handler", Fcall_with_exception_handler,
       Scall_with_exception_handler, (repv thunk, repv handler_thunk),
       rep_Subr2) /*
-::doc:call-with-exception-handler::
+::doc:rep.lang.interpreter#call-with-exception-handler::
 call-with-exception-handler THUNK HANDLER-THUNK
 
 Call THUNK and return its value. However if an exception of any form
@@ -2063,7 +2012,7 @@ and return its value.
 
 DEFUN("raise-exception", Fraise_exception,
       Sraise_exception, (repv ex), rep_Subr1) /*
-::doc:raise-exception::
+::doc:rep.lang.interpreter#raise-exception::
 raise-exception DATA
 
 Raise the exception represented by the cons cell DATA.
@@ -2084,13 +2033,42 @@ repv Fthrow (repv tag, repv value) {
 DEFSTRING(jl, ".jl");
 DEFSTRING(jlc, ".jlc");
 
+static void
+add_path (const char *env, repv var)
+{
+    repv list = Qnil;
+    char *ptr = getenv (env);
+    while (ptr != 0 && *ptr != 0)
+    {
+	char *end = strchr (ptr, ':');
+	list = Fcons (end ? rep_string_dupn (ptr, end - ptr)
+		      : rep_string_dup (ptr), list);
+	ptr = end ? end + 1 : 0;
+    }
+    Fset (var, Fnconc (rep_list_2 (Fnreverse (list),
+				   Fsymbol_value (var, Qt))));
+}
+
 void
 rep_lispcmds_init(void)
 {
     DEFSTRING (common_exec, REP_COMMON_EXEC_DIRECTORY);
+    repv tem;
 
+    tem = rep_push_structure ("rep.lang.interpreter");
     rep_ADD_SUBR(Squote);
     rep_ADD_SUBR(Slambda);
+    rep_ADD_SUBR(Scond);
+    rep_ADD_SUBR(Scall_with_exception_handler);
+    rep_ADD_SUBR(Sraise_exception);
+    rep_ADD_SUBR(Sfunctionp);
+    rep_ADD_SUBR(Smacrop);
+    rep_ADD_SUBR(Sspecial_form_p);
+    rep_ADD_SUBR(Ssubrp);
+    rep_ADD_SUBR(Ssubr_name);
+    rep_pop_structure (tem);
+
+    tem = rep_push_structure ("rep.data");
     rep_ADD_SUBR(Scar);
     rep_ADD_SUBR(Scdr);
     rep_ADD_SUBR(Slist);
@@ -2130,10 +2108,6 @@ rep_lispcmds_init(void)
     rep_ADD_SUBR(Slength);
     rep_ADD_SUBR(Scopy_sequence);
     rep_ADD_SUBR(Selt);
-    rep_ADD_SUBR(Scond);
-    rep_ADD_SUBR(Scase);
-    rep_ADD_SUBR(Sapply);
-    rep_ADD_SUBR_INT(Sload);
     rep_ADD_SUBR(Snot);
     rep_ADD_SUBR(Sequal);
     rep_ADD_SUBR(Seq);
@@ -2152,16 +2126,18 @@ rep_lispcmds_init(void)
     rep_ADD_SUBR(Slistp);
     rep_ADD_SUBR(Sstringp);
     rep_ADD_SUBR(Svectorp);
-    rep_ADD_SUBR(Sbytecodep);
-    rep_ADD_SUBR(Sfunctionp);
-    rep_ADD_SUBR(Smacrop);
-    rep_ADD_SUBR(Sspecial_form_p);
-    rep_ADD_SUBR(Ssubrp);
     rep_ADD_SUBR(Ssequencep);
-    rep_ADD_SUBR(Ssubr_name);
+    rep_pop_structure (tem);
+
+    tem = rep_push_structure ("rep.io.files");
+    rep_ADD_SUBR (Sload_file);
+    rep_ADD_SUBR (Sload_dl_file);
+    rep_ADD_SUBR_INT(Sload);
+    rep_pop_structure (tem);
+
+    tem = rep_push_structure ("rep.system");
     rep_ADD_SUBR(Scall_hook);
-    rep_ADD_SUBR(Scall_with_exception_handler);
-    rep_ADD_SUBR(Sraise_exception);
+    rep_pop_structure (tem);
 
     rep_INTERN(provide);
 
@@ -2205,27 +2181,15 @@ rep_lispcmds_init(void)
 	  Fcons (Fsymbol_value (Qdocumentation_file, Qt), Qnil));
 
     rep_INTERN_SPECIAL(load_path);
-    Fset (Qload_path, rep_list_3(Fsymbol_value (Qlisp_lib_directory, Qt),
-				 Fsymbol_value (Qsite_lisp_directory, Qt),
-				 rep_VAL(&dot)));
+    Fset (Qload_path, Fcons (Fsymbol_value (Qlisp_lib_directory, Qt),
+			     Fcons (Fsymbol_value (Qsite_lisp_directory, Qt),
+				    Fcons (rep_VAL(&dot), Qnil))));
+    add_path ("REP_LOAD_PATH", Qload_path);
 
     rep_INTERN_SPECIAL(dl_load_path);
-    Fset (Qdl_load_path, Qnil);
-    {
-	char *ptr = getenv("LD_LIBRARY_PATH");
-	while (ptr != 0 && *ptr != 0)
-	{
-	    char *end = strchr(ptr, ':');
-	    Fset (Qdl_load_path,
-		  Fcons(end ? rep_string_dupn(ptr, end - ptr)
-			: rep_string_dup(ptr), Fsymbol_value (Qdl_load_path, Qt)));
-	    ptr = end ? end + 1 : 0;
-	}
-    }
-    Fset (Qdl_load_path,
-	  Fcons (Fsymbol_value (Qexec_directory, Qt),
-		 Fcons (rep_VAL (&common_exec),
-			Fnreverse(Fsymbol_value (Qdl_load_path, Qt)))));
+    Fset (Qdl_load_path, Fcons (Fsymbol_value (Qexec_directory, Qt),
+				Fcons (rep_VAL (&common_exec), Qnil)));
+    add_path ("REP_DL_LOAD_PATH", Qdl_load_path);
 
     rep_INTERN_SPECIAL(after_load_alist);
     Fset (Qafter_load_alist, Qnil);
@@ -2237,7 +2201,7 @@ rep_lispcmds_init(void)
 
     rep_INTERN_SPECIAL(load_filename);
 
+    default_suffixes = Fcons (rep_VAL (&jl), rep_VAL (&jlc));
+    rep_mark_static (&default_suffixes);
     rep_INTERN (_load_suffixes);
-    F_structure_set (rep_structure, Q_load_suffixes, Fcons (rep_VAL (&jl),
-							    rep_VAL (&jlc)));
 }
