@@ -1,6 +1,6 @@
 /* numbers.c -- Implement the tower of numeric types
    Copyright (C) 1993, 1994, 2000 John Harper <john@dcs.warwick.ac.uk>
-   $Id: numbers.c,v 1.44 2000/11/13 19:52:02 jsh Exp $
+   $Id: numbers.c,v 1.46.2.1 2001/04/09 21:38:52 jsh Exp $
 
    This file is part of librep.
 
@@ -846,6 +846,57 @@ number_cmp (repv v1, repv v2)
     return 1;
 }
 
+static const signed int map[] = {
+    0,  1,  2,  3,  4,  5,  6,  7,		/* 0x30 -> 0x37 */
+    8,  9, -1, -1, -1, -1, -1, -1,
+    -1, 10, 11, 12, 13, 14, 15, 16,		/* 0x40 -> 0x48 */
+    17, 18, 19, 20, 21, 22, 23, 24,
+    25, 26, 27, 28, 29, 30, 31, 32,		/* 0x50 -> 0x58 */
+    33, 34, 35, 36
+};
+#define MAP_SIZE 0x2c
+
+#ifndef HAVE_GMP
+static rep_bool
+parse_integer_to_float (char *buf, u_int len, u_int radix,
+			int sign, double *output)
+{
+    double value = 0.0;
+
+    while (len-- > 0)
+    {
+	int d;
+	char c = *buf++;
+	d = toupper (c) - '0';
+	if (d < 0 || d >= MAP_SIZE)
+	    return rep_FALSE;
+	d = map [d];
+	if (d < 0 || d >= radix)
+	    return rep_FALSE;
+	value = value * radix + d;
+    }
+
+    *output = (sign < 0) ? value * -1.0 : value;
+    return rep_TRUE;
+}
+#endif
+
+#define INSTALL_LOCALE(var, type, locale)	\
+    do {					\
+	char *tem = setlocale (type, 0);	\
+	if (tem != 0)				\
+	{					\
+	    int len = strlen (tem);		\
+	    char *copy = alloca (len + 1);	\
+	    memcpy (copy, tem, len);		\
+	    copy[len] = 0;			\
+	    (var) = copy;			\
+	    setlocale (type, locale);		\
+	}					\
+	else					\
+	    (var) = 0;				\
+    } while (0)
+
 repv
 rep_parse_number (char *buf, u_int len, u_int radix, int sign, u_int type)
 {
@@ -888,15 +939,6 @@ rep_parse_number (char *buf, u_int len, u_int radix, int sign, u_int type)
 	}
 	if (bits < rep_LISP_INT_BITS)
 	{
-	    static const signed int map[] = {
-		 0,  1,  2,  3,  4,  5,  6,  7,		/* 0x30 -> 0x37 */
-		 8,  9, -1, -1, -1, -1, -1, -1,
-		-1, 10, 11, 12, 13, 14, 15, 16,		/* 0x40 -> 0x48 */
-		17, 18, 19, 20, 21, 22, 23, 24,
-		25, 26, 27, 28, 29, 30, 31, 32,		/* 0x50 -> 0x58 */
-		33, 34, 35, 36
-	    };
-#define MAP_SIZE 0x2c
 	    long value = 0;
 	    char c;
 	    if (radix == 10)
@@ -954,18 +996,35 @@ rep_parse_number (char *buf, u_int len, u_int radix, int sign, u_int type)
 # else
 		value = strtol (copy, &tail, radix);
 # endif
-		if (errno != 0)
-		    goto do_float;	/* overflow */
-		else if (*tail != 0)
+		if (errno == ERANGE)
+		{
+		    /* Overflow - parse to a double, then try to convert
+		       back to an int.. */
+		    double d;
+		    if (parse_integer_to_float (buf, len, radix, sign, &d))
+		    {
+			if (d > BIGNUM_MIN && d < BIGNUM_MAX)
+			{
+			    z->z = d;
+			    return maybe_demote (rep_VAL (z));
+			}
+			else
+			{
+			    f = make_number (rep_NUMBER_FLOAT);
+			    f->f = d;
+			    return rep_VAL (f);
+			}
+		    }
+		    else
+			goto error;
+		}
+		else if (*tail != 0 || errno != 0)
 		    goto error;		/* not all characters used */
 
-		if (sign < 0)
-		    value = -value;
-
-		z->z = value;
+		z->z = (sign < 0) ? -value : value;
 		return maybe_demote (rep_VAL (z));
 	    }
-#endif
+#endif /* !HAVE_GMP */
 	}
 
     case rep_NUMBER_RATIONAL:
@@ -1005,11 +1064,8 @@ rep_parse_number (char *buf, u_int len, u_int radix, int sign, u_int type)
 #endif
 
     case rep_NUMBER_FLOAT:
-#ifndef HAVE_GMP
-    do_float:
-#endif
 #ifdef HAVE_SETLOCALE
-	old_locale = setlocale (LC_NUMERIC, "C");
+	INSTALL_LOCALE (old_locale, LC_NUMERIC, "C");
 #endif
 	d = strtod (buf, &tem);
 #ifdef HAVE_SETLOCALE
@@ -1105,7 +1161,7 @@ rep_print_number_to_string (repv obj, int radix, int prec)
     case rep_NUMBER_FLOAT:		/* XXX handle radix arg */
 	sprintf (fmt, "%%.%dg", prec < 0 ? 16 : prec);
 #ifdef HAVE_SETLOCALE
-	old_locale = setlocale (LC_NUMERIC, "C");
+	INSTALL_LOCALE (old_locale, LC_NUMERIC, "C");
 #endif
 #ifdef HAVE_SNPRINTF
 	snprintf(buf, sizeof(buf), fmt, rep_NUMBER(obj,f));
