@@ -1,6 +1,6 @@
 /* rep_subrs.h -- mostly LISP subr declarations
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: rep_subrs.h,v 1.70 2000/11/15 19:07:56 jsh Exp $
+   $Id: rep_subrs.h,v 1.71 2001/02/20 22:27:21 jsh Exp $
 
    This file is part of Jade.
 
@@ -327,7 +327,7 @@ enum rep_message {
     rep_restore_message,
     rep_redisplay_message
 };
-void (*rep_message_fun)(enum rep_message fn, ...);
+extern void (*rep_message_fun)(enum rep_message fn, ...);
 
 /* from misc.c */
 extern u_char *rep_str_dupn(const u_char *old, int len);
