@@ -1,6 +1,6 @@
 /* streams.c -- Lisp stream handling
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: streams.c,v 1.36 2000/04/17 13:59:28 john Exp $
+   $Id: streams.c,v 1.39 2000/09/03 12:36:04 john Exp $
 
    This file is part of Jade.
 
@@ -120,7 +120,7 @@ top:
 	if (rep_FILEP(stream))
 	{
 	    if(rep_NILP(rep_FILE(stream)->name))
-		return rep_unbound_file_error(stream);
+		c = EOF;
 	    else if(rep_LOCAL_FILE_P(stream))
 		c = getc(rep_FILE(stream)->file.fh);
 	    else
@@ -487,7 +487,7 @@ rep_stream_read_esc(repv stream, int *c_p)
 }
 
 DEFUN("write", Fwrite, Swrite, (repv stream, repv data, repv len), rep_Subr3) /*
-::doc:write::
+::doc:rep.io.streams#write::
 write STREAM DATA [LENGTH]
 
 Writes DATA, which can either be a string or a character, to the stream
@@ -535,7 +535,7 @@ a string LENGTH can define how many characters to write.
 }
 
 DEFUN("read-char", Fread_char, Sread_char, (repv stream), rep_Subr1) /*
-::doc:read-char::
+::doc:rep.io.streams#read-char::
 read-char STREAM
 
 Reads the next character from the input-stream STREAM, if no more characters
@@ -549,7 +549,7 @@ are available returns nil.
 }
 
 DEFUN("peek-char", Fpeek_char, Speek_char, (repv stream), rep_Subr1) /*
-::doc:peek-char::
+::doc:rep.io.streams#peek-char::
 peek-char STREAM
 
 Returns the next character from the input-stream STREAM, *without*
@@ -569,7 +569,7 @@ characters are available returns nil.
 
 DEFUN("read-chars", Fread_chars, Sread_chars,
       (repv stream, repv count), rep_Subr2) /*
-::doc:read-chars::
+::doc:rep.io.streams#read-chars::
 read-chars STREAM COUNT
 
 Read upto COUNT characters from the input stream STREAM, returning a
@@ -605,7 +605,7 @@ that point. If no characters are read, nil will be returned.
 }
 
 DEFUN("read-line", Fread_line, Sread_line, (repv stream), rep_Subr1) /*
-::doc:read-line::
+::doc:rep.io.streams#read-line::
 read-line STREAM
 
 Read one line of text from STREAM.
@@ -637,7 +637,7 @@ Read one line of text from STREAM.
 }
 
 DEFUN("copy-stream", Fcopy_stream, Scopy_stream, (repv source, repv dest), rep_Subr2) /*
-::doc:copy-stream::
+::doc:rep.io.streams#copy-stream::
 copy-stream SOURCE-STREAM DEST-STREAM
 
 Copy all characters from SOURCE-STREAM to DEST-STREAM until an EOF is
@@ -671,7 +671,7 @@ read. Returns the number of characters copied.
 }
 
 DEFUN("read", Fread, Sread, (repv stream), rep_Subr1) /*
-::doc:read::
+::doc:rep.io.streams#read::
 read [STREAM]
 
 Reads one lisp-object from the input-stream STREAM (or the value of the
@@ -698,7 +698,7 @@ variable `standard-input' if STREAM is unspecified) and return it.
 }
 
 DEFUN("print", Fprint, Sprint, (repv obj, repv stream), rep_Subr2) /*
-::doc:print::
+::doc:rep.io.streams#print::
 print OBJECT [STREAM]
 
 First outputs a newline, then prints a text representation of OBJECT to
@@ -718,7 +718,7 @@ for `read'.
 }
 
 DEFUN("prin1", Fprin1, Sprin1, (repv obj, repv stream), rep_Subr2) /*
-::doc:prin1::
+::doc:rep.io.streams#prin1::
 prin1 OBJECT [STREAM]
 
 Prints a text representation of OBJECT to STREAM (or the contents of the
@@ -736,7 +736,7 @@ variable `standard-output') in a form suitable for `read'.
 }
 
 DEFUN("princ", Fprinc, Sprinc, (repv obj, repv stream), rep_Subr2) /*
-::doc:princ::
+::doc:rep.io.streams#princ::
 princ OBJECT [STREAM]
 
 Prints a text representation of OBJECT to STREAM (or the contents of the
@@ -755,7 +755,7 @@ are printed around strings.
 }
 
 DEFUN("format", Fformat, Sformat, (repv args), rep_SubrN) /*
-::doc:format::
+::doc:rep.io.streams#format::
 format STREAM FORMAT-STRING ARGS...
 
 Writes a string created from the format specification FORMAT-STRING and
@@ -1069,7 +1069,7 @@ exit:
 }
 
 DEFUN("make-string-input-stream", Fmake_string_input_stream, Smake_string_input_stream, (repv string, repv start), rep_Subr2) /*
-::doc:make-string-input-stream::
+::doc:rep.io.streams#make-string-input-stream::
 make-string-input-stream STRING [START]
 
 Returns a input stream, it will supply, in order, the characters in STRING,
@@ -1081,7 +1081,7 @@ starting from START (or the beginning of the string).
 }
 
 DEFUN("make-string-output-stream", Fmake_string_output_stream, Smake_string_output_stream, (void), rep_Subr0) /*
-::doc:make-string-output-stream::
+::doc:rep.io.streams#make-string-output-stream::
 make-string-output-stream
 
 Returns an output stream which will accumulate the characters written to
@@ -1092,7 +1092,7 @@ it for the use of the `get-output-stream-string' function.
 }
 
 DEFUN("get-output-stream-string", Fget_output_stream_string, Sget_output_stream_string, (repv strm), rep_Subr1) /*
-::doc:get-output-stream-string::
+::doc:rep.io.streams#get-output-stream-string::
 get-output-stream-string STRING-OUTPUT-STREAM
 
 Returns a string containing the characters written to the stream STRING-
@@ -1119,7 +1119,7 @@ return the new characters.
 
 DEFUN("input-stream-p", Finput_stream_p,
       Sinput_stream_p, (repv arg), rep_Subr1) /*
-::doc:input-stream-p::
+::doc:rep.io.streams#input-stream-p::
 input-stream-p ARG
 
 Returns t if ARG is an input stream.
@@ -1163,7 +1163,7 @@ Returns t if ARG is an input stream.
 
 DEFUN("output-stream-p", Foutput_stream_p,
       Soutput_stream_p, (repv arg), rep_Subr1) /*
-::doc:output-stream-p::
+::doc:rep.io.streams#output-stream-p::
 output-stream-p ARG
 
 Returns t if ARG is an output stream.
@@ -1213,6 +1213,7 @@ Returns t if ARG is an output stream.
 void
 rep_streams_init(void)
 {
+    repv tem = rep_push_structure ("rep.io.streams");
     rep_INTERN_SPECIAL(format_hooks_alist);
     rep_ADD_SUBR(Swrite);
     rep_ADD_SUBR(Sread_char);
@@ -1230,4 +1231,5 @@ rep_streams_init(void)
     rep_ADD_SUBR(Sget_output_stream_string);
     rep_ADD_SUBR(Sinput_stream_p);
     rep_ADD_SUBR(Soutput_stream_p);
+    rep_pop_structure (tem);
 }
