@@ -1,6 +1,6 @@
 /* symbols.c -- Lisp symbol handling
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: symbols.c,v 1.79 2000/12/04 18:02:57 jsh Exp $
+   $Id: symbols.c,v 1.80 2001/01/28 23:41:45 jsh Exp $
 
    This file is part of Jade.
 
@@ -665,7 +665,7 @@ rep_add_binding_to_env (repv env, repv sym, repv value)
 
 DEFUN("defvar", Fdefvar, Sdefvar, (repv args, repv tail_posn), rep_SF) /*
 ::doc:rep.lang.interpreter#defvar::
-defvar NAME DEFAULT-VALUE [DOC-STRING]
+defvar NAME [DEFAULT-VALUE [DOC-STRING]]
 
 Define a special variable called NAME whose standard value is DEFAULT-
 VALUE. If NAME is already bound to a value (that's not an autoload
@@ -679,7 +679,7 @@ If DOC-STRING is given, and is a string, it will be used to set the
 variable will be set (if necessary) not the local value.)
 ::end:: */
 {
-    if(rep_CONSP(args) && rep_CONSP(rep_CDR(args)))
+    if(rep_CONSP(args))
     {
 	int spec;
 	repv sym = rep_CAR(args), val;
@@ -687,7 +687,18 @@ variable will be set (if necessary) not the local value.)
 	repv tmp = Fdefault_boundp(sym);
 	if(!tmp)
 	    return rep_NULL;
-	val = rep_CAR(rep_CDR(args));
+
+	if (rep_CONSP(rep_CDR(args)))
+	{
+	    val = rep_CADR(args);
+	    args = rep_CDDR (args);
+	}
+	else
+	{
+	    val = Qnil;
+	    args = Qnil;
+	}
+
 	need_to_eval = rep_TRUE;
 	if(!rep_NILP(tmp))
 	{
@@ -781,9 +792,9 @@ variable will be set (if necessary) not the local value.)
 	    rep_SYM(sym)->car |= rep_SF_WEAK_MOD;
 	}
 
-	if(rep_CONSP(rep_CDR(rep_CDR(args))))
+	if(rep_CONSP(args))
 	{
-	    repv doc = rep_CAR(rep_CDR(rep_CDR(args)));
+	    repv doc = rep_CAR(args);
 	    if (rep_STRINGP (doc))
 	    {
 		if (Fput(sym, Qdocumentation, doc) == rep_NULL)
@@ -793,7 +804,7 @@ variable will be set (if necessary) not the local value.)
 	return sym;
     }
     else
-	return rep_signal_missing_arg(rep_CONSP(args) ? 2 : 1);
+	return rep_signal_missing_arg (1);
 }
 
 DEFUN("symbol-value", Fsymbol_value, Ssymbol_value, (repv sym, repv no_err), rep_Subr2) /*
